from django import forms
from django.contrib.auth import get_user_model
from .models import Room

class MultiUserField(forms.Field):
    users = []

    def get_user(self, username):
        try:
            return get_user_model().objects.get(username=username)
        except:
            return None

    def to_python(self, value):
        if not value:
            return []
        return list(value.replace(",", " ").split())

    def get_users(self):
        return self.users

    def validate(self, usernames):
        if len(usernames) > 10:
            raise ValidationError("Only 10 moderators can be added to a room by yourself. If you need more contact an administrator at admin@fslab.de.")

        failed = []
        users = []
        for username in usernames:
            user = self.get_user(username)
            if user is None:
                failed.append(username)
            else:
                users.append(user)

        if failed:
            raise ValidationError('User%s not found: %s\nPlease note: FB02 user accounts have to sign in at least once to be created on the platform!' % ('s' if len(failed) > 1 else '', ", ".join(failed)))
        else:
            self.users = users

class RoomForm(forms.ModelForm):
    other_moderators = MultiUserField(label='Other moderators (optional; usernames separated by whitespace and/or commata)', required=False, widget=forms.TextInput(attrs={'class': 'w3-input w3-border w3-light-grey'}))

    class Meta:
        model = Room
        fields = ['name', 'reason', 'other_moderators']
        widgets = {
            'name': forms.TextInput(attrs={'class': 'w3-input w3-border w3-light-grey', 'autocomplete': 'off'}),
            'reason': forms.Textarea(attrs={'class': 'w3-input w3-border w3-light-grey', 'autocomplete': 'off'}),
        }
        labels = {
            'reason': 'Approval reason (leave empty if you have been given a whitelisting pattern)',
        }

    def __init__(self, *args, **kwargs):
        initial = {}
        if 'instance' in kwargs:
            initial["other_moderators"] = ", ".join(list(l.username for l in kwargs['instance'].moderators.all() if l.username != kwargs['request'].user.username))

        request = kwargs['request']
        del kwargs['request']

        forms.ModelForm.__init__(self, *args, **kwargs, initial = initial)
        if 'instance' in kwargs:
            del self.fields['name']
            del self.fields['reason']
