from django.shortcuts import render, redirect, get_object_or_404
from django.contrib.auth.decorators import login_required
from django.contrib.auth import get_user_model
from django.core.mail import EmailMessage
from .forms import RoomForm
from .models import Room, Pattern
import re
from .modapi import modapi_get_room, modapi_reset_password, modapi_grant_moderator, modapi_revoke_moderator
from videos.models import Course

@login_required
def rooms(request):
    context = {}
    context['rooms'] = request.user.jitsi_moderating.all().order_by("name")
    context['patterns'] = request.user.jitsi_whitelisted.all().order_by("pattern")
    if request.user.is_staff:
        context['admin_rooms'] = Room.objects.all()
        context['admin_patterns'] = Pattern.objects.all()

    return render(request, "jitsi/rooms.html", context=context)

@login_required
def newroom(request):
    if request.method == "POST":
        form = RoomForm(request.POST, request=request)
        if form.is_valid():
            room = Room()
            room.name = form.cleaned_data["name"]
            room.reason = form.cleaned_data["reason"]
            room.save()

            room.moderators.add(request.user)
            for user in form.fields["other_moderators"].get_users():
                if user != request.user:
                    room.moderators.add(user)

            autoapproved = False
            for pattern in request.user.jitsi_whitelisted.all():
                if re.match("^%s$" % pattern.pattern, room.name):
                    autoapproved = True
                    room.approved = True
            room.save()

            for admin in get_user_model().objects.filter(is_superuser=True, is_active=True):
                if not admin.admin_mails:
                    continue
                subject = 'created (pattern-approved)' if autoapproved else 'requested'
                msg = "Hello %s,\n\na new Jitsi room has been %s:\nhttps://lectures.fslab.de/admin/jitsimod/room/%d/change/\nName: %s\nUser: %s\nReason:\n%s" % (admin.first_name, subject, room.id, room.name, request.user.username, room.reason)
                email = EmailMessage('lectures.fslab.de – Admin – Jitsi Room %s' % subject, msg, to=[admin.email])
                try:
                    email.send()
                except:
                    traceback.print_exc()
                    pass

            if autoapproved:
                return redirect("/jitsi/{}".format(room.id))
            else:
                return redirect("/jitsi?requested=1")
    else:
        form = RoomForm(request=request)

    return render(request, 'jitsi/newroom.html', {'form': form})

@login_required
def room(request, room_id):
    room = get_object_or_404(Room, id=room_id)
    if not room.approved or (not room.moderators.filter(username=request.user.username).exists() and not request.user.is_staff):
        return render(request, "forbidden.html")

    try:
        recordings = list(Course.objects.get(owner__username="jitsi-recordings").video_set.filter(description=room.name).order_by("name"))
    except:
        recordings = []

    updated = request.GET.get("saved") is not None
    form = None
    if request.method == "POST":
        if request.POST.get("grant"):
            modapi_grant_moderator(room.name, request.POST.get("grant"))
            return redirect("/jitsi/{}?saved=1".format(room.id))
        elif request.POST.get("revoke"):
            modapi_revoke_moderator(room.name, request.POST.get("revoke"))
            return redirect("/jitsi/{}?saved=1".format(room.id))
        elif request.POST.get("resetpassword") is not None:
            modapi_reset_password(room.name)
            return redirect("/jitsi/{}?saved=1".format(room.id))
        elif request.POST.get("deleterecording") is not None:
            todelete = []
            for recording in recordings:
                if str(recording.id) == str(request.POST.get("deleterecording")):
                    todelete.append(recording)

            for recording in todelete:
                recordings.remove(recording)
                recording.delete()

        elif request.POST.get("saveroom") is not None:
            form = RoomForm(request.POST, instance=room, request=request)
            if form.is_valid():
                room = form.save(commit=False)
                moderators = form.fields["other_moderators"].get_users()
                for user in room.moderators.all():
                    if user not in moderators and user.username != request.user.username:
                        room.moderators.remove(user)
                for user in moderators:
                    if user not in room.moderators.all():
                        room.moderators.add(user)
                room.save()

                return redirect("/jitsi/{}?saved=1".format(room.id))

    if form is None:
        form = RoomForm(instance=room, request=request)

    modapi_details = modapi_get_room(room.name)

    return render(request, "jitsi/room.html", context={'room': room, 'form': form, 'updated': updated, 'modapi_details': modapi_details, 'recordings': recordings})

