from django.template.loader import render_to_string
from django.http import HttpResponseForbidden, HttpResponse, JsonResponse
from django.views.decorators.csrf import csrf_exempt
from django.conf import settings
from django.db import transaction
from .models import Course, Worker, Video, VideoQuality
from django.shortcuts import get_object_or_404
import json
import os
import urllib

def get_worker_script(worker_key):
    return render_to_string('worker.py', {'worker_key': worker_key, 'cdn_password': settings.CDN_PASSWORD, 'endpoint': 'https://lectures.fslab.de'})

def authenticate_worker(view_func):
    def wrap(request, *args, **kwargs):
        try:
            key = request.GET.get('key')
            request.worker = Worker.objects.get(key=key, enabled=True)
            request.worker.save()
        except:
            return HttpResponseForbidden()
        return view_func(request, *args, **kwargs)
    return wrap

@authenticate_worker
def heartbeat(request):
    return HttpResponse("ok")

@authenticate_worker
def update(request):
    return HttpResponse(get_worker_script(request.worker.key), content_type='text/plain')

@authenticate_worker
def task(request):
    task = {'type': 'idle'}
    with transaction.atomic():
        unprocessed_videos = Video.objects.select_for_update(nowait=False).filter(worker=None, failed=False)
        if unprocessed_videos:
            video = unprocessed_videos[0]
            video.worker = request.worker
            video.save()
            task['type'] = 'master'
            task['course'] = video.course.id
            task['video'] = video.id
            task['notranscode'] = video.notranscode
            task['path'] = str(video.original)
    return JsonResponse(task)

@csrf_exempt
@authenticate_worker
def log(request):
    if request.method != "POST":
        return HttpResponse("ok boomer")

    req = json.loads(request.body)
    if req['type'] == 'master':
        v = Video.objects.get(id=req['video'])
        if not v.log:
            v.log = ""
        v.log += "\n".join(req['lines']) + "\n"
        if 'state' in req and req['state']:
            v.state = req['state']
        if 'failed' in req and req['failed']:
            v.state = "Error in master processing"
            v.failed = True
        if 'width' in req:
            v.width = req['width']
        if 'height' in req:
            v.height = req['height']
        if 'length' in req:
            v.length = float(req['length'])
        v.save()

    return HttpResponse("ok")

@csrf_exempt
@authenticate_worker
def upload(request):
    if request.POST.get("type") == "master":
        v = Video.objects.get(id=request.POST.get("video"))
        v.masterfile = request.FILES.get("videofile")
        if not v.thumbnail and request.FILES.get("thumbnail"):
            v.thumbnail = request.FILES.get("thumbnail")
        v.save()
    else:
        vq = VideoQuality.objects.get(quality=request.POST.get("quality"), video__id=request.POST.get("video"))
        vq.videofile = request.FILES.get("videofile")
        vq.save()

    return HttpResponse("ok")

@csrf_exempt
@authenticate_worker
def createquality(request):
    if request.method != "POST":
        return HttpResponse("ok boomer")

    req = json.loads(request.body)

    existing = VideoQuality.objects.filter(id=req['video'], quality=req['quality'])
    if not existing:
        videoquality = VideoQuality()
        videoquality.video = Video.objects.get(id=req['video'])
        videoquality.quality = req['quality']
    else:
        videoquality = existing[0]
        videoquality.failed = False

    if 'master' in req and req['master']:
        videoquality.worker = request.worker

    videoquality.command = req['command']
    videoquality.save()

    return HttpResponse("ok")

def check_key(request):
    request_uri = request.headers.get("X-Original-URI").split("?", 1)

    path = request_uri[0].lstrip("/").split('/')
    args = urllib.parse.parse_qs(request_uri[1])
    key = args['key'][0]

    course = get_object_or_404(Course, id=path[0])

    try:
        videoid = int(path[1])
    except:
        videoid = None

    if videoid is not None:
        video = get_object_or_404(Video, id=videoid)
        if video.key != key:
            return HttpResponseForbidden("check_key: invalid video key")
    elif course.key != key:
        return HttpResponseForbidden("check_key: invalid course key")

    return HttpResponse("ok")
