from django import forms
from .models import Course, Term
from django.core.exceptions import ValidationError
import traceback
from django.contrib.auth import get_user_model

class MultiUserField(forms.Field):
    users = []

    def get_user(self, username):
        try:
            return get_user_model().objects.get(username=username)
        except:
            return None

    def to_python(self, value):
        if not value:
            return []
        return list(value.replace(",", " ").split())

    def get_users(self):
        return self.users

    def validate(self, usernames):
        if len(usernames) > 10:
            raise ValidationError("Only 10 lecturers can be added to a course by yourself. If you need more contact an administrator at admin@fslab.de.")

        failed = []
        users = []
        for username in usernames:
            user = self.get_user(username)
            if user is None:
                failed.append(username)
            else:
                users.append(user)

        if failed:
            raise ValidationError('User%s not found: %s\nPlease note: FB02 user accounts have to sign in at least once to be created on the platform!' % ('s' if len(failed) > 1 else '', ", ".join(failed)))
        else:
            self.users = users

class CourseForm(forms.ModelForm):
    other_lecturers = MultiUserField(label='Other lecturers (optional; usernames separated by whitespace and/or commata)', required=False, widget=forms.TextInput(attrs={'class': 'w3-input w3-border w3-light-grey'}))

    class Meta:
        model = Course
        fields = ['term', 'title', 'description', 'accessibility', 'other_lecturers', 'students', 'owner', 'hidden']
        widgets = {
            'title': forms.TextInput(attrs={'class': 'w3-input w3-border w3-light-grey', 'autocomplete': 'off'}),
            'description': forms.Textarea(attrs={'class': 'w3-input w3-border w3-light-grey', 'autocomplete': 'off'}),
            'accessibility': forms.Select(attrs={'class': 'w3-input w3-border w3-light-grey', 'autocomplete': 'off'}),
            'term': forms.Select(attrs={'class': 'w3-input w3-border w3-light-grey', 'autocomplete': 'off'}),
            'owner': forms.Select(attrs={'class': 'w3-input w3-border w3-light-grey', 'autocomplete': 'off'}),
            'students': forms.Textarea(attrs={'class': 'w3-input w3-border w3-light-grey', 'autocomplete': 'off'}),
            'hidden': forms.CheckboxInput(attrs={'class': 'w3-check w3-border w3-light-grey', 'autocomplete': 'off'}),
        }
        labels = {
            'hidden': 'Hide course from list (Admin-Field)',
            'owner': 'Owner (Admin-Field)'
        }

    def __init__(self, *args, **kwargs):
        initial = {}
        if 'instance' in kwargs:
            initial["other_lecturers"] = ", ".join(list(l.username for l in kwargs['instance'].lecturers.all()))
        else:
            initial["term"] = Term.objects.get(name='SS2020')

        request = kwargs['request']
        del kwargs['request']

        forms.ModelForm.__init__(self, *args, **kwargs, initial = initial)

        if 'instance' in kwargs and kwargs['instance'].term.archived:
            self.fields['term'].disabled = True
        else:
            self.fields['term'].queryset = Term.objects.filter(archived=False)

        if not request.user.is_staff:
            self.fields['hidden'].disabled = True
            self.fields['owner'].disabled = True
            del self.fields['owner']
