from django.db import models
from django.contrib.auth import get_user_model
from django.conf import settings
import secrets
import datetime
import os
import re
import time

def random_key(i):
    key = ""
    for i in range(i):
        key += secrets.choice("abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789")
    return key

def random_key_20():
    return random_key(20)

def random_key_50():
    return random_key(50)

class CDNHost(models.Model):
    name = models.CharField(max_length=200, default="", null=False, blank=False)
    enabled = models.BooleanField(default=True, null=False, blank=False)
    healthy = models.BooleanField(default=True, null=False, blank=False)
    key = models.CharField(null=False, blank=False, max_length=50, default=random_key_50, unique=True)
    url = models.TextField(max_length=200, null=False, blank=False)

    last_seen = models.DateTimeField(blank=True, null=True)
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    def __str__(self):
        return self.name

class Term(models.Model):
    name = models.CharField(max_length=1000)
    year = models.IntegerField(default=datetime.datetime.now().year)
    archived = models.BooleanField(default=False)

    def __str__(self):
        return self.name

class Course(models.Model):
    owner = models.ForeignKey(get_user_model(), related_name='owns', blank=True, null=True, on_delete=models.SET_NULL)
    lecturers = models.ManyToManyField(get_user_model(), related_name='teaching', blank=True)
    students = models.TextField('Whitelisted students (optional; usernames separated by whitespace and/or commata)', blank=True, null=True)
    views = models.IntegerField(default=0)

    key = models.CharField(blank=False, null=False, default=random_key_20, max_length=30, unique=True)
    term = models.ForeignKey(Term, on_delete=models.SET_NULL, null=True)
    title = models.CharField('Course Name', max_length=50, blank=False, null=False)
    description = models.TextField('Course Description', blank=True, null=False)
    accessibility_choices = [
        ('closed', 'Only accessible by whitelisted students and lecturers'),
        ('internal', 'Accessible by any student and lecturer'),
        ('public', 'Accessible by anybody'),
    ]
    accessibility = models.CharField('Access restrictions', max_length=8, choices=accessibility_choices, null=False, blank=False, default='closed')
    hidden = models.BooleanField('Hide Course', default=True, null=False, blank=False)
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    def is_lecturer(self, user):
        if not user.is_authenticated:
            return False
        elif self.is_admin(user):
            return True
        elif self.lecturers.filter(username=user.username).exists():
            return True
        else:
            return False

    def is_admin(self, user):
        if not user.is_authenticated:
            return False
        elif user == self.owner:
            return True
        elif user.is_staff:
            return True
        else:
            return False

    def get_students(self):
        if self.students:
            return self.students.replace(",", " ").split()
        else:
            return []

    def access_allowed(self, user):
        if self.accessibility == 'public':
            return True
        elif self.is_admin(user):
            return True
        elif self.is_lecturer(user):
            return True
        elif user.is_authenticated and user.username in self.get_students():
            return True
        elif user.is_authenticated and self.accessibility == 'internal':
            return True
        else:
            return False

    def course_path(self):
        path = str(self.id)
        return path

    def __str__(self):
        return self.title

class CourseAttachment(models.Model):
    course = models.ForeignKey(Course, on_delete=models.CASCADE)
    name = models.CharField(max_length=1000, blank=True)
    description = models.TextField(blank=True)
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    def upload_path(self, filename):
        path = os.path.join(self.course.course_path(), "attachments", filename)
        return path

    attachment = models.FileField(upload_to=upload_path)
    def filename(self):
        return os.path.basename(self.attachment.name)

    def delete(self):
        if self.attachment:
            self.attachment.delete()
        super(CourseAttachment, self).delete()

class Worker(models.Model):
    name = models.CharField(null=False, blank=False, max_length=50)
    key = models.CharField(null=False, blank=False, max_length=50, default=random_key_50, unique=True)
    enabled = models.BooleanField(default=True, null=False, blank=False)
    notes = models.TextField(blank=True,null=True)
    last_seen = models.DateTimeField(auto_now=True)

    def __str__(self):
        return self.name

class MetaVideo(models.Model):
    state = models.CharField('State', default='Waiting for processing worker... (This will normaly take only a few seconds)', null=False, blank=False, max_length=100)
    worker = models.ForeignKey(Worker, null=True, on_delete=models.PROTECT, blank=True)
    log = models.TextField('Log', null=True, blank=True)
    failed = models.BooleanField(default=False)
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    width = models.IntegerField(null=True, blank=True)
    height = models.IntegerField(null=True, blank=True)

    class Meta:
        abstract = True

class Video(MetaVideo):
    course = models.ForeignKey('Course', Course, null=False, blank=False)
    key = models.CharField(blank=False, null=False, default=random_key_20, max_length=30, unique=True)
    views = models.IntegerField(default=0)
    notranscode = models.BooleanField(default=False)
    name = models.CharField(max_length=1000, blank=True)
    description = models.TextField(blank=True)
    length = models.DecimalField(null=True, blank=True, decimal_places=3, max_digits=10)

    def clean_filename(self):
        if isinstance(self, str):
            toclean = self
        else:
            toclean = self.course.title + " - " + (self.name if self.name else str(self.id)) + os.path.splitext(self.masterfile.name)[1]
        return re.sub(r'[^A-Za-z_0-9,.\- ]', '', toclean).replace(' ', '%20')

    def original_upload_path(self, filename):
        extension = filename[::-1].split('.', 1)[0][::-1]
        return os.path.join(self.course.course_path(), 'originals', '{}.{}'.format(int(time.time()), extension))

    def subtitles_upload_path(self, filename):
        path = os.path.join(self.course.course_path(), str(self.id), 'subtitles.vtt')
        realpath = os.path.join(settings.MEDIA_ROOT, path)
        if os.path.exists(realpath):
            os.unlink(realpath)
        return path

    def master_upload_path(self, filename):
        path = os.path.join(self.course.course_path(), str(self.id), 'master.mp4')
        realpath = os.path.join(settings.MEDIA_ROOT, path)
        if os.path.exists(realpath):
            os.unlink(realpath)
        return path

    def thumb_upload_path(self, filename):
        extension = filename[::-1].split('.', 1)[0][::-1]
        path = os.path.join(self.course.course_path(), str(self.id), 'thumbnail.{}'.format(extension))
        realpath = os.path.join(settings.MEDIA_ROOT, path)
        if os.path.exists(realpath):
            os.unlink(realpath)
        return path

    def delete(self):
        if self.masterfile:
            self.masterfile.delete()
        if self.thumbnail:
            self.thumbnail.delete()
        if self.original:
            self.original.delete()
        super(Video, self).delete()

    original = models.FileField(upload_to=original_upload_path)
    masterfile = models.FileField(upload_to=master_upload_path, blank=True, null=True)
    thumbnail = models.ImageField(upload_to=thumb_upload_path, blank=True, null=True)
    subtitles = models.FileField(upload_to=subtitles_upload_path, blank=True, null=True)

class VideoAttachment(models.Model):
    video = models.ForeignKey(Video, on_delete=models.CASCADE)
    name = models.CharField(max_length=1000, blank=True)
    description = models.TextField(blank=True)
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    def upload_path(self, filename):
        path = os.path.join(self.video.course.course_path(), str(self.video.id), "attachments", filename)
        return path

    attachment = models.FileField(upload_to=upload_path)
    def filename(self):
        return os.path.basename(self.attachment.name)

    def delete(self):
        if self.attachment:
            self.attachment.delete()
        super(VideoAttachment, self).delete()

class VideoQuality(MetaVideo):
    video = models.ForeignKey(Video, on_delete=models.CASCADE)
    command = models.TextField(null=True, blank=True)
    quality = models.CharField(max_length=20, null=False, blank=False)

    def upload_path(self, filename):
        return os.path.join(self.video.course.course_path(), str(self.video.id), "{}.mp4".format(self.quality.lower()))

    videofile = models.FileField(upload_to=upload_path)

    def delete(self):
        if self.videofile:
            self.videofile.delete()
        super(VideoQuality, self).delete()

