#!/usr/bin/env python3

import requests
import subprocess
import json
import time
import datetime
import shlex
import urllib
import urllib.request
import tempfile
import os
import traceback
import base64

SERVER = '{{ endpoint }}'
WORKER_KEY = '{{ worker_key }}'
CDN_PASSWORD = '{{ cdn_password }}'

def log(task, line, **kwargs):
    task['lines'] = [str(datetime.datetime.now()) + " " + line]
    if 'state' in kwargs and kwargs['state'] == True:
        kwargs['state'] = line
    for arg, value in kwargs.items():
        task[arg] = value
    requests.post("{}/api/log?key={}".format(SERVER, WORKER_KEY), json=task)
    print(line)

def render_master(task, tmpfiles=[]):
    log(task, "Loading video file for master processing", state=True)

    origfileurl = "{}/uploads/{}".format(SERVER, task['path'])

    manager = urllib.request.HTTPPasswordMgrWithDefaultRealm()
    manager.add_password(None, SERVER, 'cdn', CDN_PASSWORD)
    authHandler = urllib.request.HTTPBasicAuthHandler(manager)
    opener = urllib.request.build_opener(authHandler)
    urllib.request.install_opener(opener)
    urllib.request.urlretrieve(origfileurl, tmpfiles[0][1])

    meta = json.loads(subprocess.check_output(["ffprobe", "-v", "quiet", "-print_format", "json", "-show_format", "-show_streams", tmpfiles[0][1]]))

    video_streams = list([stream for stream in meta['streams'] if stream['codec_type'] == 'video'])
    audio_streams = list([stream for stream in meta['streams'] if stream['codec_type'] == 'audio'])

    # basic check for existing video/audio streams
    if len(video_streams) == 0:
        log(task, "No video streams found, aborting.", failed=True)
        return
    elif len(video_streams) > 2:
        log(task, "More than two video streams found, currently not supported, aborting.", failed=True)
        return
    else:
        log(task, "Found %d video streams" % len(video_streams))

    if len(audio_streams) == 0:
        log(task, "No audio streams found, aborting.", failed=True)
        return
    else:
        log(task, "Found %d audio streams" % len(audio_streams))

    # check that all video streams have the same resolution
    resolution = (video_streams[0]['width'], video_streams[0]['height'])
    for video_stream in video_streams:
        if video_stream['width'] != resolution[0] or video_stream['height'] != resolution[1]:
            log(task, "Mismatching resolution of video streams, refusing to encode.", failed=True)
            return

    log(task, "Video resolution: {}x{}".format(*resolution), width=resolution[0], height=resolution[1])

    if 'duration' in meta['format']:
        secs = meta['format']['duration']
        log(task, "Video length: {} seconds".format(secs), length=secs)

    log(task, "Rendering master file (this will take a while)", state=True)

    ffmpeg_command  = ['ffmpeg', '-y', '-v', 'info', '-nostats']
    ffmpeg_command += ['-analyzeduration', '20000000']
    ffmpeg_command += ['-i', tmpfiles[0][1]]
    ffmpeg_command += ['-max_muxing_queue_size', '1024']

    normalize_audio = False # disabled for now

    if task['notranscode']:
        log(task, "notranscode flag is set, only remuxing video+audio")
        ffmpeg_command += ['-c:v:0', 'copy', '-map', '0:v:0']
        if len(video_streams) == 2:
            ffmpeg_command += ['-c:v:1', 'copy', '-map', '0:v:1']
        for i in range(len(audio_streams)):
            ffmpeg_command += ['-c:a:%d' % i, 'copy', '-map', '0:a:%d' % i]
    else:
        log(task, "notranscode flag is not set, transcoding video+audio")
        filters = ['[0:v:0] hqdn3d,pad=ceil(iw/2)*2:ceil(ih/2)*2 [v]']

        if normalize_audio:
            for i in range(len(audio_streams)):
                filters += ["[0:a:{i}] dynaudnorm='p=0.35:r=1:f=300',loudnorm='i=-16.0:lra=12.0:tp=-3.0',anull [amix_{i}]".format(i=i)]
        if len(video_streams) == 2:
            filters += ['[0:v:1] pad=ceil(iw/2)*2:ceil(ih/2)*2 [s]']
        ffmpeg_command += ['-filter_complex', '; '.join(filters)]
        for i in range(len(audio_streams)):
            if normalize_audio:
                ffmpeg_command += ['-map', '[amix_{}]'.format(i)]
            else:
                ffmpeg_command += ['-map', '0:a:{}'.format(i)]
            ffmpeg_command += ['-c:a:{}'.format(i), 'aac', '-b:a:{}'.format(i), '128k', '-ar:a:{}'.format(i), '48000']
        ffmpeg_command += ['-map', '[v]']
        ffmpeg_command += ['-c:v:0', 'libx264', '-threads', '4', '-pix_fmt', 'yuv420p', '-crf:v:0', '23', '-profile:v:0', 'high', '-level:v:0', '4.1', '-disposition:v:0', 'default', '-metadata:s:v:0', 'title="Video"']
        if len(video_streams) == 2:
            ffmpeg_command += ['-map', '[s]']
            ffmpeg_command += ['-c:v:1', 'libx264', '-threads', '4', '-pix_fmt', 'yuv420p', '-crf:v:1', '23', '-profile:v:1', 'high', '-level:v:1', '4.1', '-tune:v:1', 'stillimage', '-disposition:v:1', 'descriptions', '-metadata:s:v:1', 'title="Slides"']
#    ffmpeg_command += ['-aspect', '16:9']
    ffmpeg_command += ['-movflags', 'faststart', '-f', 'mp4', tmpfiles[1][1]]
    log(task, repr(ffmpeg_command))

    process = subprocess.Popen(ffmpeg_command, stdout=subprocess.PIPE, stderr=subprocess.PIPE, stdin=subprocess.PIPE)
    process.poll()

    while process.poll() is None:
        if process.poll() is not None:
            break
        try:
            progress = subprocess.check_output(["progress", "-p", str(process.pid)]).decode().replace("\t", "").splitlines()[1].split()[0]
            log(task, "Estimated rendering progress: {}".format(progress), state=True)
        except:
            pass
        time.sleep(10)

    stdout, stderr = process.communicate()
    log(task, stderr.decode())
    if process.poll() != 0:
        log(task, "Error rendering master file, see log above.", failed=True)
        return

    upload_cmd = ["curl", "-v", "{}/api/upload?key={}".format(SERVER, WORKER_KEY)]
    upload_cmd += ["-F", "type=master"]
    upload_cmd += ["-F", "video={}".format(task['video'])]
    upload_cmd += ["-F", "videofile=@{}".format(tmpfiles[1][1])]

    try:
        log(task, "Generating thumbnail image...")
        thumbnail_command = ['ffmpeg', '-y', '-v', 'info', '-nostats']
        thumbnail_command += ['-analyzeduration', '20000000']
        thumbnail_command += ['-i', tmpfiles[1][1]]
        thumbnail_command += ['-vframes', '1', '-an', '-ss', '5']
        thumbnail_command += [tmpfiles[2][1]]
        process = subprocess.Popen(thumbnail_command, stdout=subprocess.PIPE, stderr=subprocess.PIPE, stdin=subprocess.PIPE)
        stdout, stderr = process.communicate()
        log(task, stderr.decode())
        if process.poll() == 0 and os.path.getsize(tmpfiles[2][1]) > 0:
            upload_cmd += ["-F", "thumbnail=@{}".format(tmpfiles[2][1])]
        else:
            raise Exception("Non-Zero exit status from ffmpeg")
    except:
        log(task, "Error generating thumbnail image, ignoring.")

    log(task, "Uploading master to server...", state=True)
    process = subprocess.Popen(upload_cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, stdin=subprocess.PIPE)
    stdout, stderr = process.communicate()
    if process.poll() == 0:
        log(task, "Master published successfully :)", state=True)
    else:
        log(task, "Upload failed...", failed=True)
        print(stderr.decode())
        raise Exception("Error uploading video...")

while True:
    try:
        task = json.loads(requests.get("{}/api/task?key={}".format(SERVER, WORKER_KEY)).text)
    except:
        traceback.print_exc()
        time.sleep(10)
        continue

    if task['type'] == 'idle':
        print("Nothing to do, waiting 10 seconds.")
        time.sleep(10)
    elif task['type'] == 'master':
        extension = task['path'][::-1].split('.', 1)[0][::-1]
        tmpfiles = [
            tempfile.mkstemp(suffix=".{}".format(extension)),
            tempfile.mkstemp(suffix=".mp4"),
            tempfile.mkstemp(suffix=".jpg")
        ]
        try:
            render_master(task, tmpfiles)
            for i in range(len(tmpfiles)):
                try:
                    os.close(tmpfiles[i][0])
                    os.unlink(tmpfiles[i][1])
                except:
                    traceback.print_exc()
        except:
            error = traceback.format_exc()
            log(task, "An error occured during processing", failed=True)
            log(task, error)
            log(task, "Recovery-Files for manual processing by admin:")
            log(task, " Source: {}".format(tmpfiles[0][1]))
            log(task, " Destination: {}".format(tmpfiles[1][1]))
            log(task, " Thumbnail: {}".format(tmpfiles[2][1]))
    else:
        print("Unknown task, ignoring")
