from django.shortcuts import render, redirect, get_object_or_404
from django.http import HttpResponseForbidden, HttpResponse, JsonResponse
from django.contrib.auth.decorators import login_required
from django.db.models import F
from django.conf import settings
from django.views.decorators.clickjacking import xframe_options_exempt
from django.contrib.auth import get_user_model
from django.core.mail import EmailMessage
from .models import Term, Course, Video, VideoQuality, CourseAttachment, VideoAttachment, CDNHost, Worker
import json
import random
import os
import traceback
from .forms import CourseForm

@login_required
def index(request):
    attending_courses = []
    public_courses = []
    lecturing_courses = []
    owned_courses = []
    admin_courses = []

    workers = []
    if request.user.is_superuser:
        for worker in Worker.objects.all().order_by('name'):
            jobs = Video.objects.filter(worker=worker, masterfile="", failed=False)
            if jobs.exists():
                worker.job = jobs[0]
            else:
                worker.job = None
            workers.append(worker)

    for course in Course.objects.all().order_by('created_at'):
        if course.owner == request.user:
            owned_courses.append(course)
        elif course.lecturers.filter(username=request.user.username).exists():
            lecturing_courses.append(course)
        elif not course.hidden and course.accessibility in ['public', 'internal']:
            public_courses.append(course)
        elif not course.hidden and request.user.username in course.get_students():
            attending_courses.append(course)
        elif request.user.is_staff:
            admin_courses.append(course)

    return render(request, "index.html", context={'attending_courses': attending_courses, 'owned_courses': owned_courses, 'public_courses': public_courses, 'lecturing_courses': lecturing_courses, 'admin_courses': admin_courses, 'workers': workers})

@login_required
def newcourse(request):
    if request.method == "POST":
        form = CourseForm(request.POST, request=request)
        if form.is_valid():
            course = Course()
            course.title = form.cleaned_data["title"]
            course.description = form.cleaned_data["description"]
            course.accessibility = form.cleaned_data["accessibility"]
            course.term = form.cleaned_data["term"]
            course.students = form.cleaned_data["students"]
            course.owner = request.user
            course.save()

            for user in form.fields["other_lecturers"].get_users():
                if user != request.user:
                    course.lecturers.add(user)
            course.save()

            for admin in get_user_model().objects.filter(is_superuser=True, is_active=True):
                if not admin.admin_mails:
                    continue
                msg = "Hello %s,\n\na new course has been created:\nhttps://lectures.fslab.de/course/%d\nOwner: %s\nTitle: %s\nDescription:\n%s" % (admin.first_name, course.id, request.user.username, course.title, course.description)
                email = EmailMessage('lectures.fslab.de – Admin – Course creation', msg, to=[admin.email])
                try:
                    email.send()
                except:
                    traceback.print_exc()
                    pass

            return redirect("/course/{}".format(course.id))
    else:
        form = CourseForm(request=request)

    return render(request, 'newcourse.html', {'form': form})

def course(request, course_id):
    course = get_object_or_404(Course, id=course_id)
    if not course.access_allowed(request.user):
        if not request.user.is_authenticated:
            return redirect('%s?next=%s' % (settings.LOGIN_URL, request.path))
        else:
            return render(request, "forbidden.html")

    Course.objects.filter(id=course.id).update(views=F('views')+1)

    is_lecturer = course.is_lecturer(request.user)
    is_admin = course.is_admin(request.user)

    preview = True if request.GET.get("preview") == "yes" else False
    if preview:
        is_lecturer = False
        is_admin = False

    if is_lecturer:
        videos = course.video_set.all().order_by('name')
    else:
        videos = course.video_set.filter(failed=False).exclude(masterfile="").order_by('name')

    if is_admin:
        if request.method == "POST" and request.POST.get("savecourse") is not None:
            form = CourseForm(request.POST, instance=course, request=request)
            if form.is_valid():
                course = form.save(commit=False)
                lecturers = form.fields["other_lecturers"].get_users()
                for user in course.lecturers.all():
                    if user not in lecturers:
                        course.lecturers.remove(user)
                for user in lecturers:
                    if user not in course.lecturers.all():
                        course.lecturers.add(user)
                course.save()

                return redirect("/course/{}?saved=1".format(course.id))
        else:
            form = CourseForm(instance=course, request=request)
        form.is_saved = request.GET.get("saved") is not None
    else:
        form = None
    attachments = course.courseattachment_set.all()

    return render(request, "course.html", context={'course': course, 'videos': videos, 'attachments': attachments, 'is_admin': is_admin, 'is_lecturer': is_lecturer, 'preview': preview, 'form': form})

def video_by_key(request, video_key):
    video = get_object_or_404(Video, key=video_key)
    Video.objects.filter(id=video.id).update(views=F('views')+1)
    attachments = video.videoattachment_set.all()

    return render(request, "video-direct.html", context={'course': course, 'video': video, 'attachments': attachments})

def video(request, course_id, video_id):
    course = get_object_or_404(Course, id=course_id)
    if not course.access_allowed(request.user):
        if not request.user.is_authenticated:
            return redirect('%s?next=%s' % (settings.LOGIN_URL, request.path))
        else:
            return render(request, "forbidden.html")

    video = get_object_or_404(course.video_set, id=video_id)
    Video.objects.filter(id=video.id).update(views=F('views')+1)

    attachments = video.videoattachment_set.all()
    is_lecturer = course.is_lecturer(request.user)
    is_admin = course.is_admin(request.user)
    preview = True if request.GET.get("preview") == "yes" else False
    if preview:
        is_lecturer = False
        is_admin = False

    return render(request, "video.html", context={'course': course, 'video': video, 'attachments': attachments, 'is_admin': is_admin, 'is_lecturer': is_lecturer, 'preview': preview, 'updated': request.GET.get("updated") is not None, 'autoplay': True})

@xframe_options_exempt
def embed(request):
    video = get_object_or_404(Video, key=request.GET.get('key'))
    return render(request, "embed.html", context={'video': video, 'autoplay': request.GET.get("autoplay") == "1"})

def cdn(request, path):
    hosts = CDNHost.objects.filter(enabled=True, healthy=True)
    if not hosts:
        raise Exception("No CDN hosts available!")
    host = random.choice(hosts).url.rstrip('/')
    arg_filename = '&filename={}'.format(Video.clean_filename(request.GET.get('filename'))) if (request.GET.get('filename') is not None) else ''
    redir = redirect("{}/{}?key={}&timestamp={}{}".format(host, path, request.GET.get('key'), request.GET.get('timestamp'), arg_filename))
    if path.endswith(".vtt"):
        redir["Content-Type"] = "text/vtt; charset=utf-8";
    redir["Access-Control-Allow-Origin"] = "https://lectures.fslab.de"
    redir["Access-Control-Allow-Credentials"] = "true"
    redir["Access-Control-Allow-Methods"] = "GET, HEAD, OPTIONS"
    return redir

@login_required
def upload(request):
    t = request.POST.get('type')

    if t in ['courseattachment', 'video']:
        course = get_object_or_404(Course, id=request.POST.get('course'))
    elif t in ['videoattachment']:
        video = get_object_or_404(Video, id=request.POST.get('video'))
        course = video.course
    else:
        return HttpResponseForbidden("upload: invalid type")

    if not course.is_lecturer(request.user):
        return render(request, "forbidden.html")

    if t == 'courseattachment':
        la = CourseAttachment()
        la.course = course
        la.attachment = request.FILES.get('attachment')
        if not la.attachment:
            return HttpResponse("File missing")
        la.name = request.POST.get("name").strip()
        la.description = request.POST.get("description").strip()
        la.save()
        return redirect("/course/{}".format(course.id))
    elif t == 'videoattachment':
        va = VideoAttachment()
        va.video = video
        va.attachment = request.FILES.get('attachment')
        if not va.attachment:
            return HttpResponse("File missing")
        va.name = request.POST.get("name").strip()
        va.description = request.POST.get("description").strip()
        va.save()
        return redirect("/course/{}/{}".format(course.id, video.id))
    elif t == 'video':
        v = Video()
        v.course = course
        v.name = request.POST.get("name").strip()
        v.description = request.POST.get("description").strip()
        v.original = request.FILES.get('videofile')
        v.notranscode = (request.POST.get("notranscode") == "1")
        if not v.original:
            return HttpResponse("File missing")
        v.save()
        return redirect("/course/{}".format(course.id))

@login_required
def update(request):
    t = request.POST.get('type')

    if t == 'courseattachment':
        la = get_object_or_404(CourseAttachment, id=request.POST.get('attachment'))
        course = la.course
    elif t == 'video':
        video = get_object_or_404(Video, id=request.POST.get('video'))
        course = video.course
    elif t == 'videoattachment':
        va = get_object_or_404(VideoAttachment, id=request.POST.get('attachment'))
        video = va.video
        course = video.course
    else:
        return HttpResponseForbidden("update: unknown data type")

    if not course.is_lecturer(request.user):
        return render(request, "forbidden.html")

    if t == 'courseattachment':
        la.name = request.POST.get("name").strip()
        la.description = request.POST.get("description").strip()
        la.save()
        return redirect("/course/{}".format(course.id))
    elif t == 'video':
        video.name = request.POST.get("name").strip()
        video.description = request.POST.get("description").strip()
        if request.FILES.get("subtitles"):
            video.subtitles = request.FILES.get("subtitles")
        if request.FILES.get("thumbnail"):
            video.thumbnail = request.FILES.get("thumbnail")
        video.save()
        return redirect("/course/{}/{}?updated".format(course.id, video.id))
    elif t == 'videoattachment':
        va.name = request.POST.get("name").strip()
        va.description = request.POST.get("description").strip()
        va.save()
        return redirect("/course/{}/{}".format(course.id, video.id))

@login_required
def delete(request):
    t = request.POST.get('type')

    if t == 'courseattachment':
        la = get_object_or_404(CourseAttachment, id=request.POST.get('attachment'))
        course = la.course
    elif t == 'video':
        video = get_object_or_404(Video, id=request.POST.get('video'))
        course = video.course
    elif t == 'videoattachment':
        va = get_object_or_404(VideoAttachment, id=request.POST.get('attachment'))
        video = va.video
        course = video.course
    else:
        return HttpResponseForbidden("delete: invalid type")

    if not course.is_lecturer(request.user):
        return render(request, "forbidden.html")

    if t == 'courseattachment':
        la.delete()
        return redirect("/course/{}".format(course.id))
    elif t == 'video':
        video.delete()
        return redirect("/course/{}".format(course.id))
    elif t == 'videoattachment':
        va.delete()
        return redirect("/course/{}/{}".format(course.id, video.id))

@login_required
def getlog(request):
    video = get_object_or_404(Video, id=request.GET.get('video'))
    course = video.course
    if not course.is_lecturer(request.user):
        return render(request, "forbidden.html")

    return HttpResponse(video.log, content_type='text/plain')
